@@ -112,7 +112,7 @@
          (s   (mem-stream-open in)))
     (let loop ((total 0))
       (define bv (make-bytevector 7))
-      (let ((read (false-if-exception (stream-read! s bv))))
+      (let ((read (false-if-end-of-stream (stream-read! s bv))))
         (if (not read)
             (begin
               (stream-close s)
@@ -520,6 +520,18 @@
          (eq? (object-class f)
               (lookup-class "zlib_zip_filter")))))
 
+(test-assert "make-zip-filter"
+  (let* ((c (lookup-class "zlib_zip_filter"))
+         (f (make-zip-filter c)))
+    (and (filter? f)
+         (eq? (object-class f) c))))
+
+(test-assert "make-unzip-filter"
+  (let* ((c (lookup-class "zlib_unzip_filter"))
+         (f (make-unzip-filter c)))
+    (and (filter? f)
+         (eq? (object-class f) c))))
+
 (test-assert "stacked filtered streams"
   (let* ((bv  (make-random-bytevector 5555))
          (in  (mem-stream-open bv))
@@ -546,6 +558,17 @@
          (store-close fs)
          r)))))
 
+(test-assert "stream->port error reported"
+  (let* ((s (mem-stream-open #vu8(1 2 3 4)))
+         (f (filtered-stream-open s (make-unzip-filter
+                                     (lookup-class "zlib_unzip_filter")))))
+    (catch 'chop-error
+      (lambda ()
+        (pk 'no-error (read (stream->port f)))
+        #f)
+      (lambda (key err . args)
+        (not (= err error/stream-end))))))
+
 (test-end)
 
 
