@@ -1,5 +1,5 @@
 # libchop -- a utility library for distributed storage and data backup
-# Copyright (C) 2010  Ludovic Courtès <ludo@gnu.org>
+# Copyright (C) 2010, 2011  Ludovic Courtès <ludo@gnu.org>
 #
 # Libchop is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +25,9 @@ fi
 
 set -x
 
+LANG=C
+export LANG
+
 chop_fail_if()
 {
     if eval "$@"
